unit ShellUtils;

interface

uses Windows, ShlObj;

type
  TContextMenu = class(TObject)
    ContextMenu2: IContextMenu2;
    Menu: HMENU;
    Parent: HWND;
    constructor Create(Parent: HWND; const Path: WideString);
    destructor Destroy; override;
    procedure HandleMenuMsg(var Message);
    procedure Popup(SX, SY: Integer);
  end;

procedure SetWallpaper(const Path: WideString; Tile: Boolean);

implementation

uses Messages, ActiveX, ShellAPI, NSys, NLib;

procedure SetWallpaper(const Path: WideString; Tile: Boolean);
var
  key: HKEY;
  tileflag: string;
begin
  if Tile then
    tileflag := '1'
  else
    tileflag := '0';
  if RegOpenKeyEx(HKEY_CURRENT_USER, 'Control Panel\desktop', 0, KEY_WRITE,
    key) = ERROR_SUCCESS then
  begin
    RegSetValueEx(key, 'TileWallPaper', 0, REG_SZ, PChar(tileflag),
        Length(tileflag)+1); // +1 for null character
    RegCloseKey(key);
  end;
  SystemParametersInfoW(SPI_SETDESKWALLPAPER, 0, PWideChar(Path),
      SPIF_UPDATEINIFILE or SPIF_SENDWININICHANGE);
end;

constructor TContextMenu.Create(Parent: HWND; const Path: WideString);
var
  ma: IMalloc;
  dir, fname: WideString;
  desktop, folder: IShellFolder;
  len, attr: ULONG;
  dpidl, pidl: PItemIdList;
  cm: IContextMenu;
  r: HRESULT;
begin
  Self.Parent := Parent;
  Menu := CreatePopupMenu;
  dpidl := nil;
  pidl := nil;
  dir := DirName(path);
  fname := BaseName(path);
  CoGetMalloc(1, ma);
  SHGetDesktopFolder(desktop);
  try
    r := desktop.ParseDisplayName(0, nil, PWideChar(dir), len, dpidl, attr);
    if FAILED(r) then Exit;
    r := desktop.BindToObject(dpidl, nil, IID_IShellFolder, folder);
    if FAILED(r) then Exit;
    r := folder.ParseDisplayName(0, nil, PWideChar(fname), len, pidl, attr);
    if FAILED(r) then Exit;
    folder.GetUIObjectOf(Parent, 1, pidl, IID_IContextMenu, nil, cm);
    ContextMenu2 := cm as IContextMenu2;
    ContextMenu2.QueryContextMenu(Menu, 0, 1, $7fff, CMF_EXPLORE);
  finally
    ma.Free(dpidl);
    ma.Free(pidl);
  end;
end;

destructor TContextMenu.Destroy;
begin
  ContextMenu2 := nil;
  DestroyMenu(Menu);
end;

procedure TContextMenu.Popup(SX, SY: Integer);
var
  id: Integer;
  info: TCMInvokeCommandInfo;
begin
  id := Integer(TrackPopupMenu(Menu,
      TPM_LEFTALIGN or TPM_RIGHTBUTTON or TPM_RETURNCMD, SX, SY, 0, Parent,
      nil));
  if Id <> 0 then
  begin
    FillChar(info, SizeOf(Info), 0);
    info.cbSize := SizeOf(info);
    info.lpVerb := MAKEINTRESOURCE(id-1);
    info.nShow := SW_SHOWNORMAL;
    info.hWnd := Parent;
    ContextMenu2.InvokeCommand(info);
  end;
end;

procedure TContextMenu.HandleMenuMsg(var Message);
begin
  if Assigned(ContextMenu2) then
    with TMessage(Message) do
      case Msg Of
        WM_INITMENUPOPUP, WM_DRAWITEM, WM_MEASUREITEM:
          ContextMenu2.HandleMenuMsg(Msg, wParam, lParam);
      end;
end;

initialization
  OleInitialize(nil);

finalization
  OleUnInitialize;

end.
